package de.fhdw.gaming.memory.impl;

import java.util.Objects;

import de.fhdw.gaming.core.domain.Player;
import de.fhdw.gaming.core.domain.Strategy;
import de.fhdw.gaming.memory.Key;

/**
 * A Key object that identifies something by a combination of two players and a strategy.
 */
public class MemoryKeyImpl implements Key {
    /**
     * Represents the player that "owns" the key.
     */
    private final Player<?> player;
    /**
     * Represents the Player-Object that the player  plays against.
     */
    private final Player<?> opponent;
    /**
     * Represents the strategy that the player uses.
     */
    private final Strategy<?, ?, ?> strategy;
    
    /**
     * Creates a MemoryKeyImpl object with the given parameters.
     * @param player
     * @param opponent
     * @param strategy
     */
    public MemoryKeyImpl(Player<?> player, Player<?> opponent, Strategy<?, ?, ?> strategy) {
        this.player = player;
        this.opponent = opponent;
        this.strategy = strategy;
    }
    
    /**
     * Returns the player "remembering" the outcomes. The strategy saved here belongs to this player.
     */
    @Override
    public Player<?> getPlayer() {
        return player;
    }
    
    /**
     * Returns the opponent of the active player.
     */
    @Override
    public Player<?> getOpponent() {
        return opponent;
    }
    
    /**
     * Returns the strategy of the active player, used against the opponent.
     */
    @Override
    public Strategy<?, ?, ?> getStrategy() {
        return strategy;
    }

    /**
     * Compares if two keys are equal depending on the entire strategy and the name of the players.
     * Otherwise a different outcome would lead to the player not being recognised.
     */
    @Override
    public boolean equals(Object obj) {
        if (obj == null || this.getClass() != obj.getClass()) { 
            return false; 
        }
        final MemoryKeyImpl other = (MemoryKeyImpl) obj;

        return Objects.equals(player.getName(), other.player.getName()) 
                && Objects.equals(opponent.getName(), other.opponent.getName()) 
                && Objects.equals(strategy, other.strategy);
    }
    
    @Override
    public int hashCode() {
        return Objects.hash(player.getName(), opponent.getName(), strategy);
    }
}
